syntax = "proto3";

message LogEntry {
	string source = 1;
	int64 time_nano = 2;
	bytes line = 3;
	bool partial = 4;
	PartialLogEntryMetadata partial_log_metadata = 5;
}

message PartialLogEntryMetadata {
	bool last = 1;
	string id = 2;
	int32 ordinal = 3;
}

