/*
Package asm provides the API for streaming assembly and disassembly of tar
archives.

Using the `github.com/vbatts/tar-split/tar/storage` for Packing/Unpacking the
metadata for a stream, as well as an implementation of Getting/Putting the file
entries' payload.
*/
package asm
