syntax = "proto3";

package containerd.types;

import weak "gogoproto/gogo.proto";

option go_package = "github.com/containerd/containerd/api/types;types";

// Platform follows the structure of the OCI platform specification, from
// descriptors.
message Platform {
	string os = 1 [(gogoproto.customname) = "OS"];
	string architecture = 2;
	string variant = 3;
}
