package daemon // import "github.com/docker/docker/daemon"

import (
	"github.com/docker/docker/container"
)

func (daemon *Daemon) saveAppArmorConfig(container *container.Container) error {
	return nil
}
