// Copyright 2017 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package socket

const (
	sysRECVMMSG = 0x10ef
	sysSENDMMSG = 0x10f7
)
