syntax = "proto3";

package containerd.events;

import "google/protobuf/any.proto";
import weak "gogoproto/gogo.proto";
import weak "github.com/containerd/containerd/protobuf/plugin/fieldpath.proto";

option go_package = "github.com/containerd/containerd/api/events;events";
option (containerd.plugin.fieldpath_all) = true;

message ContainerCreate {
	string id = 1;
	string image = 2;
	message Runtime {
		string name = 1;
		google.protobuf.Any options = 2;
	}
	Runtime runtime = 3;
}

message ContainerUpdate {
	string id = 1;
	string image = 2;
	map<string, string> labels  = 3;
	string snapshot_key = 4;
}

message ContainerDelete {
	string id = 1;
}
