syntax = "proto3";

package command_line;

service CommandLine {
  rpc Echo(CommandRequest) returns (CommandResponse) {}
}

message CommandRequest {
  string text = 1;
}

message CommandResponse {
  string text = 1;
}
