/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type Attachment struct {
	Type_ string `json:"Type,omitempty"`

	Path string `json:"Path,omitempty"`

	IgnoreFlushes bool `json:"IgnoreFlushes,omitempty"`

	CachingMode string `json:"CachingMode,omitempty"`

	NoWriteHardening bool `json:"NoWriteHardening,omitempty"`

	DisableExpansionOptimization bool `json:"DisableExpansionOptimization,omitempty"`

	IgnoreRelativeLocator bool `json:"IgnoreRelativeLocator,omitempty"`

	CaptureIoAttributionContext bool `json:"CaptureIoAttributionContext,omitempty"`

	ReadOnly bool `json:"ReadOnly,omitempty"`
}
